@@ -48,6 +48,8 @@ VideoSaveOptionsPage::VideoSaveOptionsPage(QWidget *parent)
     connect(m_ui->kcfg_videoFilenameTemplate, &QLineEdit::textChanged,
             this, &VideoSaveOptionsPage::updateFilenamePreview);
 
+    m_ui->preview->setFixedHeight(m_ui->kcfg_videoFilenameTemplate->height());
+
     m_ui->videoFormatComboBox->setModel(videoFormatModel);
     const auto format = static_cast<VideoPlatform::Format>(Settings::preferredVideoFormat());
     m_ui->videoFormatComboBox->setCurrentIndex(videoFormatModel->indexOfFormat(format));
@@ -57,21 +59,15 @@ VideoSaveOptionsPage::VideoSaveOptionsPage(QWidget *parent)
     });
     connect(m_ui->videoFormatComboBox, &QComboBox::currentTextChanged, this, &VideoSaveOptionsPage::updateFilenamePreview);
 
-    QString captureInstruction = i18n(
-        "You can use the following placeholders in the filename, which will be replaced "
-        "with actual text when the file is saved:<blockquote>");
-    for (auto it = ExportManager::filenamePlaceholders.cbegin(); it != ExportManager::filenamePlaceholders.cend(); ++it) {
-        // Only show placeholders with descriptions
-        if (it->description.isEmpty()) {
-            continue;
+    m_ui->captureInstructionLabel->setText(captureInstructions(false));
+    connect(m_ui->captureInstructionLabel, &QLabel::linkActivated, this, [this](const QString &link) {
+        if (link == u"showmore"_s) {
+            m_ui->captureInstructionLabel->setText(captureInstructions(true));
+        } else if (link == u"showless"_s) {
+            m_ui->captureInstructionLabel->setText(captureInstructions(false));
+        } else {
+            m_ui->kcfg_videoFilenameTemplate->insert(link);
         }
-        captureInstruction += u"<a href=%1>%1</a>: %2<br>"_s.arg(it->htmlKey, it->description.toString());
-    }
-    captureInstruction += u"<a href='/'>/</a>: "_s + i18n("To save to a sub-folder");
-    captureInstruction += u"</blockquote>"_s;
-    m_ui->captureInstructionLabel->setText(captureInstruction);
-    connect(m_ui->captureInstructionLabel, &QLabel::linkActivated, this, [this](const QString &placeholder) {
-        m_ui->kcfg_videoFilenameTemplate->insert(placeholder);
     });
 }
 
