@@ -6,10 +6,6 @@
 
 #pragma once
 
-#include "Gui/ExportMenu.h"
-#include "Gui/HelpMenu.h"
-#include "Gui/OptionsMenu.h"
-
 #include <QQuickView>
 #include <QQmlContext>
 
@@ -22,9 +18,6 @@ class SpectacleWindowPrivate;
 class SpectacleWindow : public QQuickView
 {
     Q_OBJECT
-    Q_PROPERTY(ExportMenu *exportMenu READ exportMenu CONSTANT FINAL)
-    Q_PROPERTY(OptionsMenu *optionsMenu READ optionsMenu CONSTANT FINAL)
-    Q_PROPERTY(HelpMenu *helpMenu READ helpMenu CONSTANT FINAL)
     Q_PROPERTY(bool annotating READ isAnnotating WRITE setAnnotating NOTIFY annotatingChanged FINAL)
     Q_PROPERTY(qreal logicalX READ logicalX NOTIFY logicalXChanged)
     Q_PROPERTY(qreal logicalY READ logicalY NOTIFY logicalYChanged)
@@ -39,10 +32,6 @@ public:
         Previous,
     };
 
-    ExportMenu *exportMenu() const;
-    OptionsMenu *optionsMenu() const;
-    HelpMenu *helpMenu() const;
-
     qreal logicalX() const;
     qreal logicalY() const;
 
@@ -54,7 +43,7 @@ public:
      */
     void unminimize();
 
-    static QVector<SpectacleWindow *> instances();
+    static QList<SpectacleWindow *> instances();
 
     /**
      * Set the visibility of all SpectacleWindows created in SpectacleCore.
@@ -96,10 +85,7 @@ public Q_SLOTS:
     void showPreferencesDialog();
     void showFontDialog();
     void showColorDialog(int option);
-    // TODO: Remove in Qt6. Qt.openUrlExternally() doesn't activate the window on wayland.
-    void openUrlExternally(const QUrl &url);
     void openContainingFolder(const QUrl &url);
-    void startDrag();
 
 Q_SIGNALS:
     void annotatingChanged();
@@ -122,7 +108,7 @@ protected:
     void keyPressEvent(QKeyEvent *event) override;
     void keyReleaseEvent(QKeyEvent *event) override;
 
-    static QVector<SpectacleWindow *> s_spectacleWindowInstances;
+    static QList<SpectacleWindow *> s_spectacleWindowInstances;
     static bool s_synchronizingVisibility;
     static bool s_synchronizingTitle;
     static TitlePreset s_lastTitlePreset;
@@ -130,9 +116,8 @@ protected:
     static bool s_synchronizingAnnotating;
     static bool s_isAnnotating;
 
-    const std::unique_ptr<ExportMenu> m_exportMenu;
-    const std::unique_ptr<OptionsMenu> m_optionsMenu;
-    const std::unique_ptr<HelpMenu> m_helpMenu;
     const std::unique_ptr<QQmlContext> m_context;
     std::unique_ptr<QQmlComponent> m_component;
+
+    QKeySequence m_pressedKeys;
 };
