@@ -7,11 +7,21 @@
 
 #include "ViewerWindow.h"
 
+#include "Config.h"
 #include "SpectacleCore.h"
+#include "Gui/ExportMenu.h"
 #include "spectacle_gui_debug.h"
 
+#include <KUrlMimeData>
+#include <Kirigami/Platform/Units>
+
 #include <QApplication>
 #include <QClipboard>
+#include <QDrag>
+#include <QFile>
+#include <QMimeData>
+
+using namespace Qt::StringLiterals;
 
 ViewerWindow *ViewerWindow::s_viewerWindowInstance = nullptr;
 
@@ -21,15 +31,10 @@ ViewerWindow::ViewerWindow(Mode mode, QQmlEngine *engine, QWindow *parent)
 {
     s_viewerWindowInstance = this;
     s_isAnnotating = false;
-// QGuiApplication::paletteChanged() is deprecated in Qt 6.
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    connect(qGuiApp, &QGuiApplication::paletteChanged, this, &ViewerWindow::updateColor);
-#endif
 
-    connect(SpectacleCore::instance(), &SpectacleCore::screenCaptureUrlChanged, this, &ViewerWindow::imageSizeChanged);
-    connect(SpectacleCore::instance(), &SpectacleCore::screenCaptureUrlChanged, this, &ViewerWindow::imageDprChanged);
+    m_context->setContextObject(this); // Must be before QML is initialized
 
-    connect(m_exportMenu.get(), &ExportMenu::imageShared, this, &ViewerWindow::showImageSharedMessage);
+    connect(ExportMenu::instance(), &ExportMenu::imageShared, this, &ViewerWindow::showImageSharedMessage);
 
     // set up QML
     setResizeMode(QQuickView::SizeRootObjectToView);
@@ -55,16 +60,6 @@ ViewerWindow *ViewerWindow::instance()
     return s_viewerWindowInstance;
 }
 
-QSize ViewerWindow::imageSize() const
-{
-    return ExportManager::instance()->image().size();
-}
-
-qreal ViewerWindow::imageDpr() const
-{
-    return ExportManager::instance()->image().devicePixelRatio();
-}
-
 void ViewerWindow::setMode(ViewerWindow::Mode mode)
 {
     if (mode == Dialog) {
@@ -72,9 +67,9 @@ void ViewerWindow::setMode(ViewerWindow::Mode mode)
         QVariantMap initialProperties = {
             // Set the parent in initialProperties to avoid having
             // the parent and window be null in Component.onCompleted
-            {QStringLiteral("parent"), QVariant::fromValue(contentItem())}
+            {u"parent"_s, QVariant::fromValue(contentItem())}
         };
-        setSource(QUrl(QStringLiteral("qrc:/src/Gui/DialogPage.qml")), initialProperties);
+        setSource(QUrl("%1/Gui/DialogPage.qml"_L1.arg(SPECTACLE_QML_PATH)), initialProperties);
         auto rootItem = rootObject();
         if (!rootItem) {
             return;
@@ -98,9 +93,9 @@ void ViewerWindow::setMode(ViewerWindow::Mode mode)
         QVariantMap initialProperties = {
             // Set the parent in initialProperties to avoid having
             // the parent and window be null in Component.onCompleted
-            {QStringLiteral("parent"), QVariant::fromValue(contentItem())}
+            {u"parent"_s, QVariant::fromValue(contentItem())}
         };
-        setSource(QUrl(QStringLiteral("qrc:/src/Gui/ImageView.qml")), initialProperties);
+        setSource(QUrl("%1/Gui/ImageView.qml"_L1.arg(SPECTACLE_QML_PATH)), initialProperties);
         auto rootItem = rootObject();
         if (!rootItem) {
             return;
@@ -143,46 +138,51 @@ void ViewerWindow::updateMinimumSize()
 
 void ViewerWindow::showInlineMessage(const QString &qmlFile, const QVariantMap &properties)
 {
-    rootObject()->setProperty("inlineMessageSource", qmlFile);
-    rootObject()->setProperty("inlineMessageData", properties);
+    auto rootItem = rootObject();
+    if (!rootItem) {
+        return;
+    }
+    rootItem->setProperty("inlineMessageSource", qmlFile);
+    rootItem->setProperty("inlineMessageData", properties);
 }
 
-void ViewerWindow::showSavedScreenshotMessage(const QUrl &messageArgument)
+void ViewerWindow::showSavedMessage(const QUrl &messageArgument, bool video)
 {
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/SavedMessage.qml"),
-                      {
-                          {QLatin1String("messageArgument"), messageArgument},
-                          {QLatin1String("video"), false},
-                      });
-}
-
-void ViewerWindow::showSavedVideoMessage(const QUrl &messageArgument)
-{
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/SavedMessage.qml"),
-                      {
-                          {QLatin1String("messageArgument"), messageArgument},
-                          {QLatin1String("video"), true},
-                      });
+    showInlineMessage("%1/Gui/SavedMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                      {{"messageArgument"_L1, messageArgument}, {"video"_L1, video}});
 }
 
 void ViewerWindow::showSavedAndCopiedMessage(const QUrl &messageArgument)
 {
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/SavedAndCopiedMessage.qml"), {{QLatin1String("messageArgument"), messageArgument}});
+    showInlineMessage("%1/Gui/SavedAndCopiedMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                      {{"messageArgument"_L1, messageArgument}});
 }
 
-void ViewerWindow::showSavedAndLocationCopiedMessage(const QUrl &messageArgument)
+void ViewerWindow::showSavedAndLocationCopiedMessage(const QUrl &messageArgument, bool video)
 {
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/SavedAndLocationCopied.qml"), {{QLatin1String("messageArgument"), messageArgument}});
+    showInlineMessage("%1/Gui/SavedAndLocationCopied.qml"_L1.arg(SPECTACLE_QML_PATH),
+                      {{"messageArgument"_L1, messageArgument}, {"video"_L1, video}});
 }
 
 void ViewerWindow::showCopiedMessage()
 {
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/CopiedMessage.qml"), {});
+    showInlineMessage("%1/Gui/CopiedMessage.qml"_L1.arg(SPECTACLE_QML_PATH), {});
+}
+
+void ViewerWindow::showLocationCopiedMessage()
+{
+    showInlineMessage("%1/Gui/LocationCopiedMessage.qml"_L1.arg(SPECTACLE_QML_PATH), {});
 }
 
 void ViewerWindow::showScreenshotFailedMessage()
 {
-    showInlineMessage(QStringLiteral("qrc:/src/Gui/ScreenshotFailedMessage.qml"), {});
+    showInlineMessage("%1/Gui/ScreenshotFailedMessage.qml"_L1.arg(SPECTACLE_QML_PATH), {});
+}
+
+void ViewerWindow::showRecordingFailedMessage(const QString &messageArgument)
+{
+    showInlineMessage("%1/Gui/RecordingFailedMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                      {{"messageArgument"_L1, messageArgument}});
 }
 
 void ViewerWindow::showImageSharedMessage(int errorCode, const QString &messageArgument)
@@ -193,24 +193,87 @@ void ViewerWindow::showImageSharedMessage(int errorCode, const QString &messageA
     }
 
     if (errorCode) {
-        showInlineMessage(QStringLiteral("qrc:/src/Gui/ShareErrorMessage.qml"), {{QLatin1String("messageArgument"), messageArgument}});
+        showInlineMessage("%1/Gui/ShareErrorMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                          {{"messageArgument"_L1, messageArgument}});
     } else {
-        showInlineMessage(QStringLiteral("qrc:/src/Gui/SharedMessage.qml"), {{QLatin1String("messageArgument"), messageArgument}});
+        showInlineMessage("%1/Gui/SharedMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                          {{"messageArgument"_L1, messageArgument}});
         if (!messageArgument.isEmpty()) {
             QApplication::clipboard()->setText(messageArgument);
         }
     }
 }
 
+void ViewerWindow::showVideoSharedMessage(int errorCode, const QString &messageArgument)
+{
+    if (errorCode == 1 || status() != QQuickView::Ready) {
+        // errorCode == 1 means the user cancelled the sharing
+        return;
+    }
+
+    if (errorCode) {
+        showInlineMessage("%1/Gui/ShareErrorMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                          {{"messageArgument"_L1, messageArgument}, {"video"_L1, true}});
+    } else {
+        showInlineMessage("%1/Gui/SharedMessage.qml"_L1.arg(SPECTACLE_QML_PATH),
+                          {{"messageArgument"_L1, messageArgument}, {"video"_L1, true}});
+        if (!messageArgument.isEmpty()) {
+            QApplication::clipboard()->setText(messageArgument);
+        }
+    }
+}
+
+void ViewerWindow::startDrag()
+{
+    if (m_mode == Dialog) {
+        return;
+    }
+
+    SpectacleCore::instance()->syncExportImage();
+    auto exportManager = ExportManager::instance();
+    const auto &image = exportManager->image();
+
+    const QUrl tempFile = SpectacleCore::instance()->videoMode() ? SpectacleCore::instance()->currentVideo() : exportManager->tempSave();
+    if (!tempFile.isValid()) {
+        return;
+    }
+
+    auto mimeData = new QMimeData;
+    mimeData->setUrls(QList<QUrl>{tempFile});
+    // "application/x-kde-suggestedfilename" is handled by KIO/PasteJob.
+    // It is only used when QMimeData::formats() is empty or when the user is
+    // prompted to set a filename for the content after drag & drop or paste.
+    // When QMimeData::formats() is empty, a dialog for picking the data format
+    // is supposed to appear.
+    // It's likely that users will never see the data format dialog with Spectacle.
+    mimeData->setData(u"application/x-kde-suggestedfilename"_s, QFile::encodeName(tempFile.fileName()));
+    KUrlMimeData::exportUrlsToPortal(mimeData);
+
+    auto dragHandler = new QDrag(this);
+    dragHandler->setMimeData(mimeData);
+
+    if (SpectacleCore::instance()->videoMode()) {
+        auto units = engine()->singletonInstance<Kirigami::Platform::Units *>("org.kde.kirigami.platform", "Units");
+        auto iconSize = units->iconSizes()->large();
+        dragHandler->setPixmap(QIcon::fromTheme(u"video-x-generic"_s).pixmap(iconSize, iconSize));
+    } else {
+        QSize size = image.size();
+        QPixmap pixmap = QPixmap::fromImage(image);
+        // TODO: use the composed pixmap with annotations instead
+        if (size.width() > 256 || size.height() > 256) {
+            dragHandler->setPixmap(pixmap.scaled(256, 256, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        } else {
+            dragHandler->setPixmap(pixmap);
+        }
+    }
+    dragHandler->exec(Qt::CopyAction);
+}
+
 bool ViewerWindow::event(QEvent *event)
 {
-// This should work in Qt 5, but doesn't.
-// The event type simply never happens in response to color scheme changes.
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     if (event->type() == QEvent::ApplicationPaletteChange) {
         updateColor();
     }
-#endif
     return SpectacleWindow::event(event);
 }
 
@@ -253,3 +316,5 @@ void ViewerWindow::keyReleaseEvent(QKeyEvent *event)
         }
     }
 }
+
+#include "moc_ViewerWindow.cpp"
