@@ -6,6 +6,7 @@
  */
 
 #include "Config.h"
+#include "ShortcutActions.h"
 #include "SpectacleCore.h"
 #include "CommandLineOptions.h"
 #include "SpectacleDBusAdapter.h"
@@ -24,31 +25,30 @@
 #include <KMessageBox>
 #include <KWindowSystem>
 
+using namespace Qt::StringLiterals;
+
 int main(int argc, char **argv)
 {
     // set up the application
 
     QCoreApplication::setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-#endif
     QApplication app(argc, argv);
 
-    KLocalizedString::setApplicationDomain("spectacle");
-    QCoreApplication::setOrganizationDomain(QStringLiteral("org.kde"));
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("spectacle"));
+    QCoreApplication::setOrganizationDomain(u"org.kde"_s);
 
-    KAboutData aboutData(QStringLiteral("spectacle"),
+    KAboutData aboutData(u"spectacle"_s,
                          i18n("Spectacle"),
                          QStringLiteral(SPECTACLE_VERSION),
                          i18n("KDE Screenshot Utility"),
                          KAboutLicense::GPL_V2,
                          i18n("(C) 2015 Boudhayan Gupta"));
-    aboutData.addAuthor(QStringLiteral("Boudhayan Gupta"), QString(), QStringLiteral("bgupta@kde.org"));
-    aboutData.addAuthor(QStringLiteral("David Redondo"), QString(), QStringLiteral("kde@david-redondo.de"));
-    aboutData.addAuthor(QStringLiteral("Noah Davis"), QString(), QStringLiteral("noahadvs@gmail.com"));
+    aboutData.addAuthor(u"Boudhayan Gupta"_s, {}, u"bgupta@kde.org"_s);
+    aboutData.addAuthor(u"David Redondo"_s, {}, u"kde@david-redondo.de"_s);
+    aboutData.addAuthor(u"Noah Davis"_s, {}, u"noahadvs@gmail.com"_s);
     aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
     KAboutData::setApplicationData(aboutData);
-    app.setWindowIcon(QIcon::fromTheme(QStringLiteral("spectacle")));
+    app.setWindowIcon(QIcon::fromTheme(u"spectacle"_s));
 
     QCommandLineParser commandLineParser;
     aboutData.setupCommandLine(&commandLineParser);
@@ -75,9 +75,6 @@ int main(int argc, char **argv)
 
     // Prevent session manager from restoring the app on start up.
     // https://bugs.kde.org/show_bug.cgi?id=430411
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QGuiApplication::setFallbackSessionManagementEnabled(false);
-#endif
     auto disableSessionManagement = [](QSessionManager &sm) {
         sm.setRestartHint(QSessionManager::RestartNever);
     };
@@ -106,20 +103,27 @@ int main(int argc, char **argv)
     SpectacleCore spectacleCore;
 
     QObject::connect(&service, &KDBusService::activateRequested, &spectacleCore, &SpectacleCore::activate);
+    QObject::connect(&service, &KDBusService::activateActionRequested, &spectacleCore, &SpectacleCore::activateAction);
 
     QObject::connect(&app, &QCoreApplication::aboutToQuit, Settings::self(), &Settings::save);
     QObject::connect(&spectacleCore, &SpectacleCore::allDone, &app, &QCoreApplication::quit, Qt::QueuedConnection);
 
     // create the dbus connections
     SpectacleDBusAdapter *dbusAdapter = new SpectacleDBusAdapter(&spectacleCore);
-    QObject::connect(&spectacleCore, &SpectacleCore::grabFailed, dbusAdapter, &SpectacleDBusAdapter::ScreenshotFailed);
+    QObject::connect(&spectacleCore, &SpectacleCore::dbusScreenshotFailed, dbusAdapter, &SpectacleDBusAdapter::ScreenshotFailed);
+    QObject::connect(&spectacleCore, &SpectacleCore::dbusRecordingFailed, dbusAdapter, &SpectacleDBusAdapter::RecordingFailed);
     QObject::connect(ExportManager::instance(), &ExportManager::imageExported,
                      &spectacleCore, [dbusAdapter](const ExportManager::Actions &actions, const QUrl &url) {
         Q_UNUSED(actions)
         Q_EMIT dbusAdapter->ScreenshotTaken(url.toLocalFile());
     });
-    QDBusConnection::sessionBus().registerObject(QStringLiteral("/"), &spectacleCore);
-    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.Spectacle"));
+    QObject::connect(ExportManager::instance(), &ExportManager::videoExported,
+                     &spectacleCore, [dbusAdapter](const ExportManager::Actions &actions, const QUrl &url) {
+        Q_UNUSED(actions)
+        Q_EMIT dbusAdapter->RecordingTaken(url.toLocalFile());
+    });
+    QDBusConnection::sessionBus().registerObject(u"/"_s, &spectacleCore);
+    QDBusConnection::sessionBus().registerService(u"org.kde.Spectacle"_s);
 
     // fire it up
     spectacleCore.activate(app.arguments(), QDir::currentPath());
